function [S,lines,labels,stats]=rqa(x,d,r,L)
%function S=rqa(x,d,r,L)
%x=data vector, d=embed dimension
%r=radius, L=lag
%[S,lines,labels,stats]=rqa(x,d,r,L)
%% Generating the Distance Matrix
    m=lagembed(x,d,L);   % This is a function written by Danny Kaplan, we could substitute our own
    DistanceMatrix=squareform(pdist(m,'euclidean')); % Got this from the '06 ATI RQA code
    [i,j]=find(abs(DistanceMatrix)<=r); %Find anything smaller or equal to the radius.
    S=sparse(i,j,1);     % Create a SPARSE matrix, S, use as the Recurrence maxtrix. 
                         % One could clear the DistanceMatrix to free memory at this
                         % point.

%% COUNTING RECURRENCE
%  Compute percent recurrence
%  get upper triangle, less the identity line.
    triangle=triu(S,1); % Extracts upper triangle from the Recurr. matrix, less the idenity line.

%% COMPUTING PERCENT RECURENCE
%  NNZ is a built-in sparse matrix function that counts the number of non-zero
%  elements in a sparse matrix
   nrecur=(nnz(S)-length(S))/2;
   
    if nrecur==0
        % If no recurrence, fill output with NaN and exit.
        S=NaN;
        lines=NaN;
        labels='precur, pdeter, trend, maxline, meanline, RelEnt';
        stats=[NaN, NaN, NaN, NaN, NaN, NaN];
        return
    else
        precur=((nnz(S)-length(S))/(length(S)^2-length(S)))*100;
    end
%% SPDIAGS extracts the diagonals of a sparse matrix.
%  Should make line counting much simpler. No need for all those confusing FOR loops.
    [B,d]=spdiags(triangle);

%% COMPUTE TREND DATA
        % returns the N in each diagonal, d is the index of each diagonal
        % Use tindex to make a percent rec. for each diagonal
    tindex=(length(S):-1:1)';
    raw_trend=zeros(length(S),1);
    raw_trend(d)=sum(B);
    raw_trend=raw_trend./tindex.*100;
        %check for empty matrix that will blow-up polyfit.
    if isempty(raw_trend)
        trend=NaN;
    else
        [p,~]=polyfit(1:floor(length(S)*.9),raw_trend(1:floor(length(S)*.9))',1);
        trend=p(1,1)*1000; %fit to 90% of length, mult coefficent by 1000.
    end

%% COMPUTING THE LINE COUNTS
minline=2; % set a minimum line length, this can become an option.

% This section finds the index of the zeros in the matrix B,
% which contains the diagonals of one triangle of the recurrence matrix
% (the idenity line excluded). The find command indexes the matrix sequentially
% from 1 to the total number of elements. The element numbers for a 2X2 matrix
% would be [1 3; 2 4]. You get a hit for every zero. If you take the
% difference of the resulting vector, minus 1, it yields the length of an
% interceding vector of ones, a line. Here is an e.g. using a row vector
% rather than a col. vector, since it types easier: B=[0 1 1 1 0], a line of
% length 3.  find(B==0) yields [1 5], diff([1 5])-1=3, the line length. So
% this solution finds line lengths in the interior of the B matrix, BUT
% fails if a line butts up against either edge of the B matrix, e.g. say
% B=[0 1 1 1 1], find(B==0) returns a 1, and you miss the line of length 4.
% A solution is to "bracket" B with a row of zeros at each top and bottom.

%"Bracket B with zeros"
B=[zeros(1,size(B,2)); B ; zeros(1,size(B,2))];
%Get list of line lengths, sorted from largest to smallest
lines=sort(diff(find(B==0))-1,'descend');

%Delete line counts less than the minimum.
lines(lines<minline)=[]; %lines(lines>200)=[]; % Can define a maximum line length too.
if isempty(lines)
    maxline=1; meanline=1;
    RelEnt=NaN; pdeter=0;
    labels='precur, pdeter, trend, maxline, meanline, RelEnt';
    stats=[precur, pdeter, trend, maxline, meanline, RelEnt];
    return
else
    maxline=max(lines);
    meanline=mean(lines);
end
    
%% COMPUTE ENTROPY, USE LOG BASE 2, DIVIDE BY MAX ENTROPY TO GET RELATIVE
%  ENTROPY
    [count,bin]=histcounts(lines,'BinLimits',[minline,maxline]);
    total=sum(count);
    p=count./total;
    p(p==0)=[];
    Entropy=-1*sum(p.*log2(p));
    % Transform to relative entropy: entropy/max entropy
    % More comparable across contexts and conditions.
    RelEnt=Entropy/(-1*log2(1/length(bin)));

%% COMPUTE PERCENT DETERMINISM
    pdeter=(sum(lines)/nrecur)*100;

%%  RETURN THE STATISTICAL MEASURES.
    labels='precur, pdeter, trend, maxline, meanline, RelEnt';
    stats=[precur, pdeter, trend, maxline, meanline, RelEnt];
% The SPY function is abuilt-in function for plotting sparse matrices.  I
% addapted it to plot recurrence matricies.
%% Make Recurrence Plot            line length histogram    
% subplot(1,2,1)
% histogram(lines,'BinLimits',[minline,maxline]);
% subplot(1,2,2)
% spy(S,'k'), axis('xy'), %figure, loglog(hist(lines,1:maxline))

string={['rec= ' num2str(precur)],
    ['det= ' num2str(pdeter)],
    ['maxline= ' num2str(maxline)],
    ['meanline= ' num2str(meanline)],
    ['trend= ' num2str(trend)]
    ['RelEnt=' num2str(RelEnt)]};
%disp(char(string))
%% Lag Embedding program.
function y = lagembed(x,M,lag)
% lagEmbed(x,dim,lag) generates an embedding of a time series on a vector
% lagEmbed(x,dim) generates an m-dimensional embedding with lag 1
% lagEmbed(x,dim,lag) uses the specified lag
if nargin < 3
	lag = 1;
end
[xr,xc] = size(x);
%Check for a matrix.
if xr && xc > 1
    error('LagEmbed requires a col. vector, matrix input.');
end
%convert x to a column
if xr == 1	
    x = x';
end
lx = length(x);	
newsize = lx - lag*(M-1);
y = zeros(newsize,M);
i=1;
for j = 0:-lag:lag*(-(M-1))
    
	first=1+lag*(M-1)+j;
	last=first+newsize-1;
    
        if last > lx
            last = lx;
        end
	y(:,i) = x(first:last, 1);
	i = i+1;
end
